@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 from __future__ import absolute_import
 
 try:
@@ -270,6 +272,9 @@ def test_attributes():
         assert 1 == f.attributes(il)[0]
         assert 20 == f.attributes(xl)[0]
 
+        assert f.tracecount == len(f.attributes(il))
+        assert iter(f.attributes(il))
+
         ils = [(i // 5) + 1 for i in range(25)]
         attrils = list(map(int, f.attributes(il)[:]))
         assert ils == attrils
@@ -471,6 +476,31 @@ def test_traces_raw():
         for raw, gen in zip(f.trace.raw[::-1], f.trace[::-1]):
             assert np.array_equal(raw, gen)
 
+def test_read_text_sequence():
+    with segyio.open('test-data/multi-text.sgy', ignore_geometry = True) as f:
+        for text in f.text[:]:
+            assert text
+
+        assert iter(f.text)
+
+@tmpfiles('test-data/multi-text.sgy')
+def test_put_text_sequence(tmpdir):
+    lines = { 1: 'first line', 10: 'last line' }
+    ref = segyio.tools.create_text_header(lines)
+    fname = str(tmpdir / 'multi-text.sgy')
+
+    with segyio.open(fname, mode = 'r+', ignore_geometry = True) as f:
+        f.text[0] = ref
+        f.text[-1] = ref
+        f.text[1:4] = [ref, ref, ref]
+
+    # ref doesn't have to be bytes for reading, but has to in order to compare
+    # with the returned object from text
+    ref = bytearray(ref, 'ascii')
+    ref = bytes(ref)
+    with segyio.open(fname, ignore_geometry = True) as f:
+        for text in f.text:
+            assert text == ref
 
 def test_read_header():
     with segyio.open("test-data/small.sgy") as f:
@@ -752,9 +782,6 @@ def test_open_fails_unstructured():
         with pytest.raises(ValueError):
             _ = f.xline[:, :]
 
-        with pytest.raises(ValueError):
-            _ = f.depth_slice[2]
-
         # operations that don't rely on geometry still works
         assert f.header[2][189] == 1
         assert (list(f.attributes(189)[:]) ==
@@ -1172,7 +1199,7 @@ def test_create_from_naught_unstructured(tmpdir):
         # Set header field 'offset' to 1 in all headers
         dst.header = {TraceField.offset: 1}
 
-    with segyio.open(tmpdir / "unstructured.sgy") as f:
+    with segyio.open(tmpdir / "unstructured.sgy", ignore_geometry=True) as f:
         assert 1 == approx(f.trace[1][0], abs=1e-4)
         assert 2 == approx(f.trace[1][1], abs=1e-4)
         assert 150 == approx(f.trace[1][-1], abs=1e-4)
@@ -1367,6 +1394,17 @@ def test_depth_slice_reading():
     with pytest.raises(IndexError):
         _ = f.depth_slice[len(f.samples)]
 
+def test_depth_slice_array_shape():
+    with segyio.open("test-data/1xN.sgy") as f:
+        shape = (len(f.fast), len(f.slow))
+        assert f.depth_slice.shape == shape
+
+def test_depth_slice_unstructured():
+    with segyio.open("test-data/small.sgy", ignore_geometry = True) as f:
+        traces = f.trace.raw[:]
+        shape = len(f.trace)
+        assert f.depth_slice.shape == shape
+        np.testing.assert_almost_equal(f.depth_slice[0], traces[:,0])
 
 @tmpfiles("test-data/small.sgy")
 def test_depth_slice_writing(tmpdir):
@@ -1463,3 +1501,12 @@ def test_missing_format_ibmfloat_fallback(tmpdir):
         with segyio.open(tmpdir / 'small.sgy') as f:
             assert int(f.format) == 1
             assert f.dtype       == np.dtype(np.float32)
+
+def test_utf8_filename():
+    with segyio.open('test-data/小文件.sgy') as f:
+        assert list(f.ilines) == [1, 2, 3, 4, 5]
+
+@tmpfiles(u'test-data/小文件.sgy')
+def test_utf8_filename_pypath(tmpdir):
+    with segyio.open(tmpdir / '小文件.sgy') as f:
+        assert list(f.ilines) == [1, 2, 3, 4, 5]
